<?php

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Router\Route;

// No direct access.
defined('_JEXEC') or die;

abstract class AbstractClubsController extends BaseController
{
    
    protected abstract function getNameOfElement();
    
    protected function getModelName()
    {
        return $this->getNameOfElement();
    }
    
    protected function getNameOfView()
    {
        return strtolower($this->getNameOfElement());
    }
    
    protected abstract function getDataMapping();
    
    function new()
    {
        $obj = call_user_func(array('Clubs' . $this->getNameOfElement(), 'create' . $this->getNameOfElement()));
        
        // Fetch the posted data
        $values = $this->loadData();

        $this->filterPreCheck($values);
        
        // Check the input data
        $error = ! $this->checkDataIsValid($values, true, Null);
        
        $view = $this->getNameOfView();
        
        if($error)
        {
            $urldata = $this->packData($values);
            $this->setRedirect(Route::_("index.php?option=com_clubs&view={$view}&id=new&data={$urldata}", false));
            return;
        }
        
        $this->applyData($obj, $values);
        
        // Do the actual work
        $obj->save();
        $this->setRedirect(Route::_("index.php?option=com_clubs&view={$view}s", false));
    }
    
    function change()
    {
        $app = Factory::getApplication();
        $input = $app->input;
        $id = (int) $input->post->getInt('id');
        
        $obj = call_user_func(array('Clubs' . $this->getNameOfElement(), 'load' . $this->getNameOfElement()), (int) $id);
        
        // Fetch the posted data
        $values = $this->loadData();
        
        $this->filterPreCheck($values);
        
        // Check the input data
        $error = ! $this->checkDataIsValid($values, false, $obj);
        
        $view = $this->getNameOfView();
        
        if($error)
        {
            $urldata = $this->packData($values);
            $this->setRedirect(Route::_("index.php?option=com_clubs&view={$view}&id={$id}&data={$urldata}", false));
            return;
        }
        
        $this->applyData($obj, $values);
        
        // Do the actual work
        $obj->save();
        $this->setRedirect(Route::_("index.php?option=com_clubs&view={$view}s", false));
    }
    
    protected function loadData()
    {
        $values = array();
        $input = Factory::getApplication()->input->post;
        
        foreach($this->getDataMapping() as $m => $f)
        {
            $filter = (isset($f['filter'])) ? $f['filter'] : 'string';
            
            $values[$m] = $input->get($m, null, $filter);
        }
        
        return $values;
    }
    
    protected function filterPreCheck(&$values){}
    
    protected function checkDataIsValid($values, bool $isNew, $obj)
    {
        $error = false;
        // Check existence of the required fields
        foreach ($this->getDataMapping() as $m => $v)
        {
            if(! isset($v['required']) || ! $v['required'])
                continue;
            
            // Field is required
            if(! $this->fieldValid($m, $values[$m], $v))
            {
                $fname = (isset($v['name'])) ? $v['name'] : $m;
                Factory::getApplication()->enqueueMessage("Das Feld $fname ist obligatorisch.", 'error');
                $error = true;
            }
        }
        
        return ! $error;
    }
    
    protected function fieldValid($name, $value, $options)
    {
        if(empty($value))
            return false;
        
        if(isset($options['filter']))
        {
            switch($options['filter'])
            {
                case 'string':
                    if(empty(trim($value)))
                        return false;
            }
        }
        
        return true;
    }
    
    private function packData($values)
    {
        // FIXME Multiple bugs: filtering not working as expected and Mapping msut be checked
        $this->filterPrePacking($values);
        
        $data = array();
        foreach($this->getDataMapping() as $m => $i)
        {
            if(isset($values[$m]))
                $data[$m] = $values[$m];
        }
        $json = json_encode($data);
        return urlencode($json);
    }
    
    protected function filterPrePacking(&$values){}
    
    public function applyData($obj, $values)
    {
        $this->applyDataToObject($obj, $values, $this->getDataMapping());
    }
    
    protected function applyDataToObject($obj, $values, $mapping)
    {
        foreach($mapping as $m => $v)
        {
            $functionName = $this->getSetterMethodName($m, $v);
            
            if($functionName === null)
            {
                continue;
            }
            
            if(isset($v['skip_null_check']))
                $value = $values[$m];
            else 
                $value = (isset($values[$m]) && strlen($values[$m]) > 0) ? $values[$m] : null;
            
            $obj->$functionName($value);
        }
    }
    
    private function getSetterMethodName($m, $options)
    {
        if(array_key_exists('setter', $options))
            return $options['setter'];
        
        $firstChar = substr($m, 0, 1);
        $restChars = substr($m, 1);
        return 'set' . strtoupper($firstChar) . $restChars;
    }
    
    function delete()
    {
        $app = Factory::getApplication();
        
        $id = $app->input->get->getInt('id');
        $name = $this->getNameOfElement();
        $app->enqueueMessage("Removal of $name with id $id.");
        
        $className = 'Clubs' . $this->getModelName();
        $functionName = 'load' . $this->getModelName();
        $element = call_user_func(array($className, $functionName), $id);
        $element->delete();
        
        $view = $this->getNameOfView();
        $this->setRedirect(Route::_("index.php?option=com_clubs&view={$view}s", false));
    }
    
}
