<?php

// No direct access.
use Joomla\CMS\Factory;

defined('_JEXEC') or die;

abstract class AbstractClubsModel
{
    protected $id;
    
    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }
    
    protected abstract function getDataMappings();
    protected abstract function getTableName();


    protected function loadData(array $data)
    {
        $this->id = $data['id'];
        
        foreach($this->getDataMappings() as $m)
            $this->$m = $data[$m];
    }
    
    protected static function loadElements(string $tableName, string $className, $where = null)
    {
        $dbo = Factory::getDbo();
        $q = $dbo->getQuery(true);
        $q->select('*')
          ->from($tableName);
        
        if(isset($where))
        {
            $q->where($where);
        }
        
        $dbo->setQuery($q);
        $dbo->execute();
        $list = $dbo->loadAssocList();
        
        $ret = array();
        foreach($list as $row)
        {
            $obj = new $className();
            $obj->loadData($row);
            
            $ret[] = $obj;
        }
        
        return $ret;
    }
    
    /**
     * @param int $id
     * @throws Exception
     * @return ClubsOffer
     */
    protected static function loadElement(int $id, string $tableName, string $className)
    {
        $dbo = Factory::getDbo();
        $q = $dbo->getQuery(true);
        
        $q->select('*')->from($tableName);
        
        $q->where('id=' . (int) $id);
        $dbo->setQuery($q);
        $dbo->execute();
        
        $row = $dbo->loadAssoc();
        
        if($row == null)
        {
            throw new Exception("No object of class $className found.");
            // TODO
        }
        
        $obj = new $className();
        $obj->loadData($row);
        $obj->loadCustomData($row);
        return $obj;
    }
    
    protected function loadCustomData($assoc)
    {}
    
    public function save()
    {
        if($this->id === 'new')
            $this->insertElement();
        else
            $this->updateElement();
    }
    
    private function insertElement()
    {
        if(! $this->isDataValid())
            throw new Exception('Data is invalid');
        
        $dbo = Factory::getDbo();
        $q = $dbo->getQuery(true);
        
        $mappings = $this->getDataMappings();
        $values = $this->getDataValues($mappings, $q);
        
        $q->insert($this->getTableName())
          ->columns(array_map(array($q, 'qn'), $mappings))
          ->values(join(',', $values))
          ;
        
        $dbo->transactionStart();
        $dbo->setQuery($q);
        $dbo->execute();
        $this->id = $dbo->insertid();
        $dbo->transactionCommit();
    }
    
    private function updateElement()
    {
        if(! $this->isDataValid())
            throw new Exception('Data is invalid');
            
        $dbo = Factory::getDbo();
        $q = $dbo->getQuery(true);
        
        $mapping = $this->getDataMappings();
        $values = $this->getDataValues($mapping, $q);
        
        $q->update($this->getTableName());
        foreach($mapping as $m)
            $q->set($q->qn($m) . '=' . $values[$m]);
        $q->where("id=". (int) $this->id);
        
        $dbo->setQuery($q);
        $dbo->execute();
    }
    
    private function getDataValues($mapping, $q)
    {
        $rawValues = array();
        
        foreach($mapping as $m)
            $rawValues[$m] = $this->$m;
            
        $this->filter($rawValues);
        
        $quotedValues = array();
        
        foreach($mapping as $m)
        {
            if(is_bool($rawValues[$m]))
                $quotedValues[$m] = $rawValues[$m] ? 'TRUE' : 'FALSE';
            else if(is_numeric($rawValues[$m]))
                $quotedValues[$m] = $rawValues[$m];
            else
                $quotedValues[$m] = $q->q($rawValues[$m]);
        }
            
        $this->postQuoteFilter($quotedValues);
        
        return $quotedValues;
    }
    
    protected function prepareDelete($dbo){}
    
    public function delete()
    {
        if($this->id === 'new')
            return;
        
        $dbo = Factory::getDbo();
        $this->prepareDelete($dbo);
        
        $q = $dbo->getQuery(true);
        $q->delete($this->getTableName())
          ->where('id=' . (int) $this->id);
        
        $dbo->setQuery($q);
        $dbo->execute();
    }
    
    protected function getRequiredDataMappings()
    {
        return $this->getDataMappings();
    }
    
    protected function isDataValid()
    {
        foreach($this->getRequiredDataMappings() as $m)
        {
            if(!isset($this->$m) || is_null($this->$m) || $this->$m === null)
                return false;
        }
        return true;
    }
    
    protected function filter(&$values){}
    protected function postQuoteFilter(&$values){}
    
    
}
