<?php

use Joomla\CMS\Factory;

// No direct access.
defined('_JEXEC') or die;

class ClubsControllerUser extends AbstractClubsController
{
    
    private function checkUserName($username, $obj = null)
    {
        $id = -1;
        
        if(!is_null($obj))
            $id = $obj->getId();
        
        return ClubsUser::isUserNameFree($username, $id);
    }
    
    protected function getNameOfElement()
    {
        return 'user';
    }

    protected function getDataMapping()
    {
        return array(
            'user'=>array('required'=>true, 'name'=>'Benutzername', 'filter'=>'cmd'),
            'pwd'=>array('required'=>false, 'name'=>'Passwort', 'filter'=>'string', 'setter'=>'setPassword'),
            'pwdConfirm'=>array('required'=>false, 'name'=>'Passwortwiederholung', 'filter'=>'string', 'setter'=>null),
            'name'=>array('required'=>true, 'name'=>'Bürgerlicher Name', 'filter'=>'string'),
            'address'=>array('required'=>true, 'name'=>'Adresse', 'filter'=>'string'),
            'city'=>array('required'=>true, 'name'=>'Stadt', 'filter'=>'string'),
            'mail'=>array('required'=>true, 'name'=>'E-Mail', 'filter'=>'string'),
            'phone'=>array('required'=>false, 'name'=>'Telefonnummer', 'filter'=>'string'),
            'mobile'=>array('required'=>false, 'name'=>'Handynummer', 'filter'=>'string')
            
        );
    }
    /**
     * {@inheritDoc}
     * @see AbstractClubsController::checkData()
     */
    protected function checkDataIsValid($values, $isNew, $obj)
    {
        if(! parent::checkDataIsValid($values, $isNew, $obj))
            return false;
        
        // TODO Auto-generated method stub
        if(isset($values['pwd']) && strlen($values['pwd']) > 0)
        {
            $pwd = $values['pwd'];
            $pwdConfirm = $values['pwdConfirm'];
            
            if(trim($pwd) != trim($pwdConfirm))
            {
                Factory::getApplication()->enqueueMessage('Die Passwörter stimmen nicht überein.', 'error');
                return false;
            }
            
            if(! ClubsUser::checkPasswordStrength(trim($pwd)))
            {
                Factory::getApplication()->enqueueMessage('Das Passwort ist zu schwach.', 'error');
                return false;
            }
            
        }
        else
        {
            if($isNew)
            {
                Factory::getApplication()->enqueueMessage('Für einen neuen Benutzer muss ein Passwort vergeben werden.', 'error');
                return false;
            }
        }
        
        if(! $this->checkUserName(trim($values['user']), $obj))
        {
            Factory::getApplication()->enqueueMessage('Username ' . $$values['user'] . ' ist nicht gültig.', 'error');
            return false;
        }
        
        return true;
    }
    
    /**
     * {@inheritDoc}
     * @see AbstractClubsController::filterPrePacking()
     */
    protected function filterPrePacking(&$values)
    {
        parent::filterPrePacking($values);
        unset($values['pwd']);
        unset($values['pwdConfirm']);
    }
    
    /**
     * {@inheritDoc}
     * @see AbstractClubsController::applyData()
     */
    public function applyData($obj, $values)
    {
        // TODO Auto-generated method stub
        $mapping = $this->getDataMapping();
        
        if(strlen($values['pwd']) == 0)
        {
            unset($values['pwd']);
            unset($mapping['pwd']);
        }
        
        $this->applyDataToObject($obj, $values, $mapping);
    }




}
