<?php

use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Factory;

// No direct access.
defined('_JEXEC') or die;

class ClubsHelperAuth
{
    
    public function checkUser($user, $pwd)
    {
        $userModel = BaseDatabaseModel::getInstance("user", "ClubsModel");
        $savedHash = $userModel->getPassword($user);
        
        return password_verify($pwd, $savedHash);
    }
    
    public function logoffCookie()
    {
        Factory::getApplication()->input->cookie->set("clubsLogin", "");
    }
    
    public function loginCookie($user, $keep)
    {
        $authCookie = array();
        $authCookie['user'] = $user;
        $authCookie['start'] = time();
        
        $value = array();
        $value['auth'] = $authCookie;
        
        $keys = $this->getKeys();
        $pkey = openssl_pkey_get_private($keys['priv']);
        $signature = '';
        openssl_sign(json_encode($authCookie), $signature, $pkey) or die("Problem signing request.");
        $value['sign'] = convert_uuencode($signature);
        
        $jsonValue = json_encode($value);
         
        $c = Factory::getApplication()->input->cookie;
        if(! $keep)
        {
            $time = 0;
            $c->set('clubsLoginKeepLoggedIn', '', 0);
        }
        else
        {
            $time = time() + 3600*24*15;
            $c->set('clubsLoginKeepLoggedIn', 'true', time() + 3600*24*356*10);
        }
        $c->set('clubsLogin', $jsonValue, $time);
    }
    
    public function checkCookie()
    {
        $cookie = Factory::getApplication()->input->cookie;
        $jsonValue = $cookie->get('clubsLogin', '', 'raw');
        
        if($jsonValue === '')
            return false;
        
        $value = json_decode($jsonValue, true);
        
        if($value === null)
            // Error in JSON code
            return false;
        
        $keys = $this->getKeys();
        $pubkey = openssl_pkey_get_public($keys['public']);
        $ret = openssl_verify(json_encode($value['auth']), convert_uudecode($value['sign']), $pubkey);
        
        if($ret == -1)
            die("Error checking signature.");
        
        if($ret == 1)
        {
            // correct signature
            $this->loginCookie($value['auth']['user'], Factory::getApplication()->input->cookie->get('clubsLoginKeepLoggedIn', false));
            return true;
        }
        else
        {
            // wrong signature
            $this->logoffCookie();
            return false;
        }
    }
    
    private function getKeys()
    {
        $keyModel = BaseDatabaseModel::getInstance('signkey', 'ClubsModel');
        return $keyModel->getKeys();
    }
    
    public function setKeepLoggedIn($keep)
    {
        $c = Factory::getApplication()->input->cookie;
        
        if($keep)
        {
            $c->set('clubsLoginKeepLoggedIn', 'true', time() + 3600*24*365*10);
        }
        else
        {
            $c->set('clubsLoginKeepLoggedIn', '', 0);
        }
    }
    
}
